/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
q3master@hendriklipka.de
*/

package de.hendriklipka.q3master.handler.heartbeat;

import de.hendriklipka.q3master.HeartbeatHandler;
import de.hendriklipka.q3master.ServerData;
import de.hendriklipka.q3master.ServerStateManager;

import java.net.InetAddress;

import org.apache.log4j.Category;

public class QuakeArenaHeartbeat implements HeartbeatHandler
{
    private Category CAT = Category.getInstance(QuakeArenaHeartbeat.class.getName());
    private Category LOG_CAT = Category.getInstance("q3master.heartbeat");

    public void handleHeartbeat(InetAddress serverAddress, int port, ServerStateManager handler)
    {
        // create server data object and set known data
        ServerData sd = new ServerData();
        sd._serverAddress = serverAddress;
        sd._serverPort = port;
        sd._lastHeartBeat = System.currentTimeMillis();
        // check for localservers (127.*)
        // they need to be replaced with their non-local IP address
        if (serverAddress.getAddress()[0] == 127)
        {
            try
            {
                CAT.debug("loopback found - trying alternate addresses for " + InetAddress.getLocalHost().getHostName());
                if (InetAddress.getLocalHost().getAddress()[0] != 127)
                {
                    CAT.debug("using local address directly - it's not loopback");
                    serverAddress = InetAddress.getLocalHost();
                    sd._serverAddress = serverAddress;
                }
                else
                {
                    InetAddress[] iaa = InetAddress.getAllByName(serverAddress.getHostName());
                    CAT.info("found " + iaa.length + " addresses");
                    for (int i = 0; i < iaa.length; i++)
                    {
                        CAT.info("alternate address: " + iaa[i].getHostAddress());
                        if (iaa[i].getAddress()[0] == 127)
                        {
                            continue;
                        }
                        CAT.debug("setting alternate address");
                        serverAddress = iaa[i];
                        sd._serverAddress = serverAddress;
                        break;
                    }
                }
            }
            catch (Exception ex)
            {
            }
            // ignore - if we cannot resolve our local address, we have bigger problems...
        }
        // send request to server and set missing data
        handler.addServer(sd);    
    }
}
