package de.hendriklipka.q3master.udpproxy;

import org.apache.log4j.Category;

import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.net.DatagramPacket;
import java.io.IOException;

public class UDPProxyThread extends Thread
{
    private static final Category CAT=Category.getInstance("proxy");
    private static final Category CAT_DBG=Category.getInstance(UDPProxyThread.class);

    private DatagramSocket _serverSocket=null;
    private InetAddress _targetAddress=null;
    private int _targetPort=0;

    private boolean _stopped=false;
    
    private int _lastLocalPort=0;
    private InetAddress _localAddress;
    
    private final char[] HEXCHARS={'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F',};

    public UDPProxyThread(String targetIP, String localIP, int port) throws SocketException, UnknownHostException
    {
        super("proxy_port_"+port);
        CAT_DBG.debug("creating proxy thread for port "+port);

        setDaemon(true);

        if (null==localIP)
            _serverSocket=new DatagramSocket(port);
        else
            _serverSocket=new DatagramSocket(port,InetAddress.getByName(localIP));
        _targetAddress=InetAddress.getByName(targetIP);
        _targetPort=port;
        _lastLocalPort=port;
        _localAddress=InetAddress.getByName("127.0.0.1");
    }

    public void stopThread()
    {
        CAT_DBG.debug("stopping thread for port "+_targetPort);
        _stopped=true;
    }

    public void run()
    {
        CAT_DBG.debug("starting thread for port "+_targetPort);
        DatagramSocket clientSocket=null;
        try
        {
//            clientSocket=new DatagramSocket(_targetPort);
//            clientSocket.setSoTimeout(50);
//            clientSocket.setReceiveBufferSize(65535);
//            clientSocket.connect(_targetAddress,_targetPort);
            while (!_stopped)
            {
                // do
                byte[] data=new byte[65535];
                DatagramPacket packet=new DatagramPacket(data,data.length,_targetAddress,_targetPort);
                try
                {
                    _serverSocket.receive(packet);
                }
                catch (IOException e)
                {
                    continue;
                }
                CAT.debug("");
                CAT.debug("received packet on port "+_targetPort+" from: "+packet.getAddress()+":"+packet.getPort());
                dumpPacket(packet);
                if (packet.getAddress().equals(_targetAddress))
                {
                    CAT.debug("packet is from target server");
                    data=packet.getData();
                    byte[] data2=new byte[packet.getLength()];
                    for (int i=0;i<data2.length;i++) {data2[i]=data[i];}
                    DatagramPacket packet2=new DatagramPacket(data2,data2.length,_localAddress,_lastLocalPort);
                    _serverSocket.send(packet2);
                    CAT.debug("sent packet packet to local server: "+_localAddress+":"+_lastLocalPort);
                }
                else
                {
                    CAT.debug("packet is from local server");
                    data=packet.getData();
                    _lastLocalPort=packet.getPort();
                    byte[] data2=new byte[packet.getLength()];
                    for (int i=0;i<data2.length;i++) {data2[i]=data[i];}
                    DatagramPacket packet2=new DatagramPacket(data2,data2.length,_targetAddress,_targetPort);
                    _serverSocket.send(packet2);
                    CAT.debug("sent packet packet to target server: "+_targetAddress+":"+_targetPort);
                }
            }
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        finally
        {
            clientSocket.close();
            _serverSocket.disconnect();
            _serverSocket.close();
        }
    }

    private void dumpPacket(DatagramPacket packet)
    {
        CAT.debug("dumping packet data");
        byte[] data=packet.getData();
        StringBuffer result=new StringBuffer();
        for (int i=0;i<packet.getLength();i++)
        {
            int c=data[i];
            dumpCharacter(c,i);
            if (c<32) c=(int)'?';
            result.append((char)c);
        }
        CAT.debug("data="+result.toString());
    }

    private void dumpCharacter(int c, int pos)
    {
        String result=""+pos+": ";
        int ch=c;
        if (ch<32) ch='?';
        result+=((char)ch)+" / ";
        int high=(c>>4)&0xf;
        result+=HEXCHARS[high];
        int low=c&0xf;
        result+=HEXCHARS[low];
        result+=" / "+c;
        CAT.debug(result);
    }
}
